import time, traceback

# Zope imports
from Globals import DTMLFile, InitializeClass
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from AccessControl.User import SimpleUser
from AccessControl.Permissions import view_management_screens, manage_users
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime

try:
    from Products.GroupUserFolder.GroupUserFolder import GroupUserFolder
    gruf_mtype = GroupUserFolder.meta_type
except:
    gruf_mtype = ''


# LDAPUserFolder package imports
from Products.LDAPUserFolder.SimpleCache import SimpleCache

addLDAPGroupFolderForm = DTMLFile('dtml/addLDAPGroupFolder', globals())


class LDAPGroupFolder(SimpleItem):
    """ """
    security = ClassSecurityInfo()

    meta_type = 'LDAPGroupFolder'
    id = 'acl_users'

    isPrincipiaFolderish=1
    isAUserFolder=1

    manage_options=(
        ({'label' : 'Groups', 'action' : 'manage_main',},)
        + SimpleItem.manage_options
        )

    security.declareProtected(view_management_screens, 'manage_main')
    manage_main = DTMLFile('dtml/groups', globals())

    
    def __setstate__(self, v):
        """ """
        LDAPGroupFolder.inheritedAttribute('__setstate__')(self, v)

        self._cache = SimpleCache()
        self._cache.setTimeout(600)
        self._cache.clear()

    def __init__( self, title, luf=''):
        """ """
        self._luf = luf
        self._cache = SimpleCache()
        self._cache.setTimeout(600)
        self._cache.clear()

    security.declareProtected(manage_users, 'getGRUF')
    def getGRUF(self):
        """ """
        return self.aq_parent.aq_parent


    security.declareProtected(manage_users, 'getLUF')
    def getLUF(self):
        """ """

        return self.getGRUF().getUserSource(self._luf)


    security.declareProtected(manage_users, 'getGroups')
    def getGroups(self, dn='*', attr=None, pwd=''):
        """ """

        return self.getLUF().getGroups(dn, attr, pwd)


    security.declareProtected(manage_users, 'getGroupType')
    def getGroupType(self, group_dn):
        """ """
        return self.getLUF().getGroupType(group_dn)

    security.declareProtected(manage_users, 'getGroupMappings')
    def getGroupMappings(self):
        """ """
        return self.getLUF().getGroupMappings()

    security.declareProtected(manage_users, 'manage_addGroupMapping')
    def manage_addGroupMapping(self, group_name, role_name, REQUEST=None):
        """ """
        self._cache.remove(group_name)
        self.getLUF().manage_addGroupMapping(group_name, role_name, None)

        if REQUEST:
            msg = 'Added LDAP group to Zope role mapping: %s -> %s' % (
                group_name, role_name)
            return self.manage_main(manage_tabs_message=msg)

    security.declareProtected(manage_users, 'manage_deleteGroupMappings')
    def manage_deleteGroupMappings(self, group_names, REQUEST=None):
        """ Delete mappings from LDAP group to Zope role """
        self._cache.clear()
        self.getLUF().manage_deleteGroupMappings(group_names, None)
        if REQUEST:
            msg = 'Deleted LDAP group to Zope role mapping for: %s' % (
                ', '.join(group_names))
            return self.manage_main(manage_tabs_message=msg)


    security.declareProtected(manage_users, 'manage_addGroup')
    def manage_addGroup( self
                       , newgroup_name
                       , newgroup_type='groupOfUniqueNames'
                       , REQUEST=None
                       ):
        """ Add a new group in groups_base """
        self.getLUF().manage_addGroup(newgroup_name, newgroup_type, None)

        if REQUEST:
            msg = 'Added new group %s' % (newgroup_name)
            return self.manage_main(manage_tabs_message=msg)


    security.declareProtected(manage_users, 'manage_deleteGroups')
    def manage_deleteGroups(self, dns=[], REQUEST=None):
        """ Delete groups from groups_base """
        self.getLUF().manage_deleteGroups(dns, None)
        self._cache.clear()
 
        if REQUEST:
            msg = 'Deleted group(s):<br> %s' % '<br>'.join(dns)
            return self.manage_main(manage_tabs_message=msg)

    security.declareProtected(manage_users, 'getUser')
    def getUser(self, name):
        """ """
        user = self._cache.get(name, '')
        if user:
            return user
        
        grps = self.getLUF().getGroups()
        for n, dn in grps:
            if n == name:
                roles = getattr(self, '_groups_mappings', {}).get(name, [])
                user = GroupUser(n, '', roles, [])
                self._cache.set(name, user)
                return user

        return None
        
    security.declareProtected(manage_users, 'getUserNames')
    def getUserNames(self):
        """ """
        return [g[0] for g in self.getLUF().getGroups()]

    security.declareProtected(manage_users, 'getUsers')
    def getUsers(self, authenticated=1):
        """ """

        data = []
        
        grps = self.getLUF().getGroups()
        for n, dn in grps:
                roles = getattr(self, '_groups_mappings', {}).get(n, [])
                user = GroupUser(n, '', roles, [])
                data.append(user)

        return data

    security.declarePrivate('_doAddUser')
    def _doAddUser(self, name, password, roles, domains, **kw):
        self.getLUF().manage_addGroup(name)
        mappings = getattr(self, '_groups_mappings', {})
        mappings[name] = roles
        self.getLUF().manage_addGroupMapping(name, self.getGroupPrefix() + name)

    security.declarePrivate('_doDelUsers')
    def _doDelUsers(self, names):
        dns = []
        luf = self.getLUF()
        for g_name, dn in luf.getGroups():
            if g_name in names:
                dns.append(dn)
        self._cache.clear()
        return luf.manage_deleteGroups(dns)

    security.declarePrivate('_doChangeUser')
    def _doChangeUser(self, name, password, roles, domains, **kw):
        """ """
        self._cache.remove(name)
        mappings = getattr(self, '_groups_mappings', {})
        mappings[name] = roles
        self._groups_mappings = mappings

        grp_prefix = self.getGroupPrefix()
        l_grp_prefix = len(grp_prefix)

        luf = self.getLUF()
        luf.manage_addGroupMapping(name, grp_prefix+name)

        # filter group roles
        new_roles = []
        for r in roles:
            if r[:l_grp_prefix] != grp_prefix:
                new_roles.append(r)
        luf._set_group_roles(name, new_roles)


def manage_addLDAPGroupFolder( self, title, luf='', REQUEST=None):
    """ """
    this_folder = self.this()

    if hasattr(aq_base(this_folder), 'acl_users') and REQUEST is not None:
        msg = 'This+object+already+contains+a+User+Folder'

    else:
        n = LDAPGroupFolder( title, luf )

        this_folder._setObject('acl_users', n)
        this_folder.__allow_groups__ = self.acl_users
        
        msg = 'Added+LDAPGroupFolder'
 
    # return to the parent object's manage_main
    if REQUEST:
        url = REQUEST['URL1']
        qs = 'manage_tabs_message=%s' % msg
        REQUEST.RESPONSE.redirect('%s/manage_main?%s' % (url, qs))


InitializeClass(LDAPGroupFolder)


class GroupUser(SimpleUser):
    """ """

    def __init__(self, name, password, roles, domains):
        SimpleUser.__init__(self, name, password, roles, domains)
        self._created = time.time()

    def getCreationTime(self):
        """ """
        return DateTime(self._created)
