# 	$Id: LDAPUserFolder.py,v 1.3 2003/10/07 04:05:53 fafhrd Exp $	
__version__='$Revision: 1.3 $'[11:-2]

# General python imports
import time, os, urllib, traceback

# Zope imports
from Globals import DTMLFile, InitializeClass
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from AccessControl.User import BasicUserFolder
from AccessControl.Permissions import view_management_screens, manage_users

from OFS.SimpleItem import SimpleItem


try:
    from Products.GroupUserFolder.GroupUserFolder import GroupUserFolder
    gruf_mtype = GroupUserFolder.meta_type
except:
    gruf_mtype = ''




# LDAPUserFolder package imports
from Products.LDAPUserFolder.LDAPUserFolder import LDAPUserFolder, EDIT_PERMISSION

try:
    from Products.CMFCore.utils import getToolByName
except:
    pass


addLDAPUserFolderExtForm = DTMLFile('dtml/addLDAPUserFolderExt', globals())


class LDAPUserFolderExt(LDAPUserFolder):
    """ """
    security = ClassSecurityInfo()

    meta_type = 'LDAPUserFolderExt'
    id = 'acl_users'

    manage_options=(
        (
        {'label' : 'Configure',	'action' : 'manage_main', 
         'help'  : ('LDAPUserFolder','Configure.stx')},
        {'label' : 'LDAP Schema', 'action' : 'manage_ldapschema',
         'help'  : ('LDAPUserFolder', 'Schema.stx')},
        {'label' : 'Caches', 'action' : 'manage_cache',
         'help'  : ('LDAPUserFolder', 'Caches.stx')},
        {'label' : 'Users', 'action' : 'manage_userrecords',
         'help'  : ('LDAPUserFolder', 'Users.stx')},
        {'label' : 'Log', 'action' : 'manage_log',
         'help'  : ('LDAPUserFolder', 'Log.stx')},
        )
        + SimpleItem.manage_options
        ) 

    security.declareProtected(view_management_screens, 'manage_main')
    manage_main = DTMLFile('dtml/properties', globals())

    security.declareProtected(EDIT_PERMISSION, 'manage_edit')
    def manage_edit( self, title, login_attr, uid_attr, users_base
                   , users_scope, roles,  groups_base, groups_scope
                   , binduid, bindpwd, binduid_usage=1, rdn_attr='cn'
                   , obj_classes='top,person', local_groups=0
                   , encryption='SHA', read_only=0, getusers_ldap = 0, REQUEST=None
                   ):
        """ Edit the LDAPUserFolder Object """

        self.getusers_ldap = not not getusers_ldap

        return LDAPUserFolder.manage_edit(
            self, title, login_attr,uid_attr, users_base
            , users_scope, roles,  groups_base, groups_scope
            , binduid, bindpwd, binduid_usage, rdn_attr
            , obj_classes, local_groups, encryption, read_only, REQUEST)


    security.declareProtected(manage_users, 'manage_addUser')
    def manage_addUser(self, REQUEST=None, kwargs={}):
        """ Add a new user record to LDAP """
        if REQUEST is None:
            req = 0
            source = kwargs
        else:
            req = 1
            source = REQUEST

        user_roles = source.get('user_roles', [])
        if user_roles:
            # map from roles to ldap groups
            # this mappings managed by LDAPGroupFolder
            new_roles = []
            for grp, val in getattr(self, '_group_roles', {}).items():
                for r in user_roles:
                    if r in val:
                        new_roles.append(grp)

            for grp, map in getattr(self, '_groups_mappings', {}).items():
                if map in user_roles:
                    new_roles.append(grp)

            source['user_roles'] = new_roles
        if req:
            return LDAPUserFolder.manage_addUser(self, source)
        else:
            return LDAPUserFolder.manage_addUser(self, None, source)
        
    def _mapRoles(self, groups):
        """ Perform the mapping of LDAP groups to Zope roles """
        mappings = getattr(self, '_groups_mappings', {})
        roles = []

        prefix=''
        try:
          acl = self.aq_parent.aq_parent
          if acl.meta_type == gruf_mtype:
            prefix = "group_"
            
        except:
          pass

        for group in groups:
            roles.append('%s%s' %(prefix, group))
            mapped_role = mappings.get(group, None)
            if mapped_role is not None and mapped_role not in roles:
                roles.append(mapped_role)

        return roles

    security.declareProtected(manage_users, 'getUsers')
    def getUsers(self, authenticated=1):
        """  """

        if getattr(self, 'getusers_ldap', 0):

            res = []
            for u in self.findUser(self._login_attr, ''):
                name = u[self._login_attr]
                res.append(self.getUser(name))
            return res

        else:
            return LDAPUserFolder.getUsers(self, authenticated)

 

    ###################################
    # Zope User management API
    ###################################
    security.declarePrivate('_doChangeUser')
    def _doChangeUser(self, name, password, roles, domains, **kw):
        """ """
        user = self.getUser(name)
        if user:
            user_dn = user.getUserDN()

            new_roles = []
            for grp, val in getattr(self, '_group_roles', {}).items():
                for r in roles:
                    if r in val:
                        new_roles.append(grp)

            for grp, map in getattr(self, '_groups_mappings', {}).items():
                if map in roles:
                    new_roles.append(grp)

            if new_roles:
                roles_dns = []
                for grp, dn in self.getGroups():
                    if grp in new_roles:
                        roles_dns.append(dn)
                self.manage_editUserRoles(user_dn, roles_dns)

            if password:
                self.manage_editUserPassword(user_dn, password)

    security.declarePrivate('_doDelUsers')
    def _doDelUsers(self, names):
        """ """
        dns = []
        login_attr = self._login_attr

        if login_attr == 'dn':
            logattr_dn = 0
        else:
            logattr_dn = 1
            users_base = self.users_base

        for name in names:
            if logattr_dn:
                dns.append('%s=%s,%s' % (login_attr, name, users_base))
            else:
                dns.append('objectClass=*,%s' % name)

        self.manage_deleteUsers(dns)
        self._v_userlistexpire = 0

    security.declarePrivate('_doAddUser')
    def _doAddUser(self, name, password, roles, domains, **kw):
        """ """
        try:
            request = self.REQUEST
        except:
            request = {'form':{}}

        args = {}

        try:
            # set default values from portal_membership
            md = getToolByName(self, 'portal_memberdata')
            for id in md.propertyIds():
                args[id] = md.getProperty(id)
        except:
            pass

        # set values from REQUEST
        for key, val in request.form.items():
            args[key] = val
            
        new_roles = []
        for grp, val in getattr(self, '_group_roles', {}).items():
            for r in roles:
                if r in val:
                    new_roles.append(grp)

        for grp, map in getattr(self, '_groups_mappings', {}).items():
            if map in roles:
                new_roles.append(grp)
            
        args.update({'user_pw':password, 'confirm_pw':password,
                     'user_roles':new_roles, self._rdnattr:name
                     })
        msg = self.manage_addUser(REQUEST=None, kwargs=args)
        self._v_userlistexpire = 0

        if msg:
            raise Exception, msg

    security.declarePrivate('_set_group_roles')
    def _set_group_roles(self, group, roles):
        """ """
        m = getattr(self, '_group_roles', {})
        m[group] = roles
        self._group_roles = m


def manage_addLDAPUserFolderExt( self, title, LDAP_server, login_attr, uid_attr 
                                 , users_base, users_scope, roles, groups_base
                                 , groups_scope, binduid, bindpwd, binduid_usage=1
                                 , rdn_attr='cn', local_groups=0, use_ssl=0
                                 , encryption='SHA', read_only=0, set_roles=0, REQUEST=None
                                 ):
    """ Called by Zope to create and install an LDAPUserFolder """
    this_folder = self.this()

    if hasattr(aq_base(this_folder), 'acl_users') and REQUEST is not None:
        msg = 'This+object+already+contains+a+User+Folder'

    else:
        n = LDAPUserFolderExt( title, LDAP_server, login_attr, uid_attr, users_base, users_scope
                               , roles, groups_base, groups_scope, binduid, bindpwd
                               , binduid_usage, rdn_attr, local_groups=local_groups
                               , use_ssl=not not use_ssl, encryption=encryption
                               , read_only=read_only, REQUEST=None
                               )
        n.set_roles = not not set_roles

        this_folder._setObject('acl_users', n)
        this_folder.__allow_groups__ = self.acl_users
        
        msg = 'Added+LDAPUserFolderExt'
 
    # return to the parent object's manage_main
    if REQUEST:
        url = REQUEST['URL1']
        qs = 'manage_tabs_message=%s' % msg
        REQUEST.RESPONSE.redirect('%s/manage_main?%s' % (url, qs))


InitializeClass(LDAPUserFolderExt)
