from AccessControl.Permissions import add_user_folders
from AccessControl import ClassSecurityInfo
from AccessControl import Permissions
from AccessControl import getSecurityManager

from Globals import InitializeClass
from Acquisition import aq_base, aq_parent, aq_inner

from Products.GroupUserFolder.GroupUserFolder  import GroupUserFolder
from Products.GroupUserFolder.GRUFUser import GRUFUser 
from Products.GroupUserFolder import class_utility


from Products.CMFPlone.MembershipTool import MembershipTool


from LDAPUserFolder import addLDAPUserFolderExtForm, \
                           manage_addLDAPUserFolderExt, \
                           LDAPUserFolderExt

from LDAPGroupFolder import addLDAPGroupFolderForm, \
                           manage_addLDAPGroupFolder, \
                           LDAPGroupFolder

import zLOG


searchForMembersMethod=None


def log(message,summary='',severity=0):
    zLOG.LOG('LDAPUserFolderExt',severity,summary,message)


def listAvailableUserSources(self, filter_permissions = 1, filter_classes = 1):
    """ Patched Original method  """

    ret = []

    # Fetch candidate types
    user = getSecurityManager().getUser()
    
    meta_types = []
    if callable(self.all_meta_types):
        all=self.all_meta_types()
    else:
        all=self.all_meta_types
        
    for meta_type in all:
        if filter_permissions and meta_type.has_key('permission'):
            if user.has_permission(meta_type['permission'],self):
                meta_types.append(meta_type)
        else:
            meta_types.append(meta_type)

    # Keep only, if needed, BasicUserFolder-derived classes
    for t in meta_types:
        if t['name'] == self.meta_type:
            continue        # Do not keep GRUF ! ;-)
        
        if filter_classes:
            try:
                if t.get('instance', None) and class_utility.isBaseClass(AccessControl.User.BasicUserFolder, t['instance']):
                    ret.append((t['name'], t['action']))
                    continue;
                    
            except:
                pass        # We ignore 'invalid' instances (ie. that wouldn't define a __base__ attribute)
                
            if t['name'].lower().find('user') > -1:
                ret.append((t['name'], t['action']))
                continue;
            
            if t['name'].lower().find('ldap') > -1:
                ret.append((t['name'], t['action']))
                continue;
            
        else:
            ret.append((t['name'], t['action']))

    return tuple(ret)


def searchForMembers( self, REQUEST=None, **kw ):
    """ Patched Original method  """
    
    if REQUEST:
        dict = REQUEST
    else:
        dict = kw

    ret = []
    names = []

    name = dict.get('name', None)
    email = dict.get('email', None)
    roles = dict.get('roles', None)
    last_login_time = dict.get('last_login_time', None)
    is_manager = self.checkPermission('Manage portal', self)

    if name:
        name = name.strip().lower()
    if not name:
        name = None
    if email:
        email = email.strip().lower()
    if not email:
        email = None

    md = self.portal_memberdata
    uf = self.acl_users
    portal = self.portal_url.getPortalObject()

#    if not(hasattr(aq_base(uf), 'getPureUsers')): # GRUF
#       return apply(searchForMembersMethod, (self, REQUEST), kw )

    for src in uf.listUserSources():
        users = None
        if src.meta_type == "LDAPUserFolderExt":
            if name:
                users = src.findUser(src._login_attr, name)
            elif email:
                users = src.findUser('mail', email)
            else:
                users = src.findUser(src._login_attr, '')
                
            if len(users) > 0 and users[0]['dn'] == 'Too many results for this query':
                continue
                
            for lu in users:
                if not lu:
                    continue        # Ignore empty users
                
                name = lu[src._login_attr]
                u = src.getUser(name=name)
                if name in names:
                    continue        # Prevent double users inclusion

                # Append user
                names.append(name)
                ret.append(GRUFUser(u, uf, source_id = src.getUserSourceId()).__of__(uf))
        else:
            users = src.getUsers()
                
            for u in users:
                if not u:
                    continue        # Ignore empty users

                name = u.getId()
                if name in names:
                    continue        # Prevent double users inclusion
                
                #if not (member.listed or is_manager):
                #    continue
                if name:
                    if (u.getUserName().lower().find(name) == -1 and
                        u.getProperty('fullname').lower().find(name) == -1):
                            continue
                if email:
                    if u.getProperty('email').lower().find(email) == -1:
                        continue
                if roles:
                    user_roles = member.getRoles()
                    found = 0
                    for r in roles:
                        if r in user_roles:
                            found = 1
                            break
                    if not found:
                        continue
                    if last_login_time:
                        if member.last_login_time < last_login_time:
                            continue
                
                # Append user
                names.append(name)
                ret.append(GRUFUser(u, uf, source_id = src.getUserSourceId()).__of__(uf))

    return ret


def initialize(context):
    context.registerClass( LDAPUserFolderExt
                         , permission=add_user_folders
                         , constructors=( addLDAPUserFolderExtForm
                                        , manage_addLDAPUserFolderExt
                                        )
                         , icon='www/ldapuserfolder.gif'
                         )

    context.registerClass( LDAPGroupFolder
                         , permission=add_user_folders
                         , constructors=( addLDAPGroupFolderForm
                                        , manage_addLDAPGroupFolder
                                        )
                         , icon='www/ldapuserfolder.gif'
                         )
 
    
    if 1:

        listAvailableUserSources.__doc__ = '\n'.join((GroupUserFolder.listAvailableUserSources.__doc__, listAvailableUserSources.__doc__))
        
        security = ClassSecurityInfo()
        security.declareProtected(Permissions.manage_users, "listAvailableUserSources")
        GroupUserFolder.listAvailableUserSources = listAvailableUserSources
        
        InitializeClass(GroupUserFolder)


        global searchForMembersMethod
        searchForMembersMethod=MembershipTool.searchForMembers
        
        searchForMembers.__doc__ = '\n'.join((MembershipTool.searchForMembers.__doc__, searchForMembers.__doc__))

        security.declarePublic( 'searchForMembers' )
        MembershipTool.searchForMembers = searchForMembers
        
        InitializeClass(MembershipTool)
        
        

# don't lose the docstrings, they are necessary if these methods
# are published directly.
#tag.__doc__ = '\n'.join((Image.tag.__doc__, tag.__doc__))
#Image.tag = tag



